/* -*- C++ -*-
*
* ---------------------------------------------------------------------
* $Id: testmarray.cpp 363 2008-07-10 15:09:44Z drory $
* ---------------------------------------------------------------------
*
* Copyright (C)  Niv Drory <drory@mpe.mpg.de>
*                         Claus A. Goessl <cag@usm.uni-muenchen.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*
* ---------------------------------------------------------------------
*
*/

#define LTL_RANGE_CHECKING
//#define LTL_USE_SIMD

#include <ltl/marray.h>
#include <ltl/statistics.h>
#include <ltl/marray_io.h>
#include <iostream>

using namespace ltl;

using std::cout;
using std::endl;

template<typename T>
void test_logic(void)
{
   MArray<T,1>A(10);
   A = 0;

   LTL_ASSERT_( !allof(A),
                "MArray allof() expression failed" );
   LTL_ASSERT_( !anyof(A),
                "MArray anyof() expression failed" );
   LTL_ASSERT_( noneof(A),
                "MArray noneof() expression failed" );
   LTL_ASSERT_( count(A)==0,
                "MArray count() expression failed" );

   A(3) = 1;
   A(7) = 1;

   LTL_ASSERT_( !allof(A),
                "MArray allof() expression failed" );
   LTL_ASSERT_( anyof(A),
                "MArray anyof() expression failed" );
   LTL_ASSERT_( !noneof(A),
                "MArray noneof() expression failed" );
   LTL_ASSERT_( count(A)==2,
                "MArray count() expression failed" );

   A = 7;
   LTL_ASSERT_( allof(A),
                "MArray allof() expression failed" );
   LTL_ASSERT_( anyof(A),
                "MArray anyof() expression failed" );
   LTL_ASSERT_( !noneof(A),
                "MArray noneof() expression failed" );
   LTL_ASSERT_( count(A)==10,
                "MArray count() expression failed" );
}

template<typename T>
void test_init(void)
{
   MArray<T,1> A(5);
   for( int i=1; i<=(int)A.nelements(); ++i)
      A(i) = T(i);

   MArray<T,1> B(5);
   B = 1,2,3,4,5;

   LTL_ASSERT( allof(A==B), "List initializer failed");
}

template<typename T>
void test_init2(void)
{
   MArray<T,1> A(5);
   A = 1,2,3,4,5;

   MArray<T,1> B;
   B = A;
   LTL_ASSERT( allof(B==A), "Assignment to uninitialized MArray failed");

   MArray<T,1> C;
   C = A+B-A;
   LTL_ASSERT( allof(C==B), "Assignment of expression to uninitialized MArray failed");
}

int main(int argc, char **argv)
{
   cerr << "Testing MArray initialization ..." << endl;

   MArray<float,2> A(5,5);
   A = 42.f;

   for( int i=1; i<=5; ++i )
      for( int j=1; j<=5; ++j )
         LTL_ASSERT_( A(i,j)==42.f, "MArray init failed" );

   cerr << "Testing MArray anyof/allof/noneof/count logic ..." << endl;
   test_logic<char>();
   test_logic<short>();
   test_logic<int>();
   test_logic<float>();
   test_logic<double>();

   cerr << "Testing MArray list initializer" << endl;
   test_init<float>();

   cerr << "Testing MArray basics ..." << endl;
   A.setBase(-1,-1);
   LTL_ASSERT_( A(-1,-1)==42.f && A(1,1)==42.f,
                "MArray setBase() failed" );
   A *= 2.0f;
   A -= A/2.f;
   cout << A << endl;
   LTL_ASSERT_( allof(A==42.f),
                "MArray basic expressions failed" );

   MArray<int, 2> I(5,5), I2(5,5);
   I = cast<int>()(A);
   I2 = 42;

   LTL_ASSERT_( allof(I == I2),
                "MArray type-cast failed" );

   MArray<float,1> B = A(-1,Range::all());
   B = 1.f;
   LTL_ASSERT_( allof(B==1.f) && allof( A(-1,Range::all())==1.f ),
                "MArray slice/allof() 1 failed" );

   MArray<float,1> C = A(Range::all(),1);
   C = 2.f;
   LTL_ASSERT_( allof(C==2.f) && allof( A(Range::all(),1)==2.f ),
                "MArray slice/allof() 2 failed" );

   A = 10.0f;
   A = log10(A);
   LTL_ASSERT_( allof(A==1.0f),
                "MArray basic mathlib expressions failed" );

   MArray<float,3> AA(10,10,10);
   AA = indexPos(AA,1) + 10*indexPos(AA,2) + 100*indexPos(AA,3);
   LTL_ASSERT_( AA(1,1,1)==111.0f && AA(6,9,2)==296.0f && AA(9,9,9)==999.0f ,
                "MArray indexPos() failed" );

   MArray<float,2> BB = AA(Range(2,7,2),5,Range(3,8,3));
   MArray<float,2> CC(3,2);
   CC = 352.f,354.f,356.f,
        652.f,654.f,656.f;
   LTL_ASSERT_( allof(BB==CC),
                "MArray subarray failed" );

   MArray<float,1> DD = BB(2,Range::all());
   MArray<float,1> EE(2);
   EE = 354.f,654.f;
   LTL_ASSERT_( allof(DD==EE),
                "MArray complex slice/allof() failed" );

   DD = 555.f;
   LTL_ASSERT_( allof(DD==555.f),
                "MArray complex slice assign failed" );
   LTL_ASSERT_( AA(4,5,3)==555.f && AA(4,5,6)==555.f,
                "MArray complex slice assign failed" );

   cerr << "Testing Assignment to uninitialized arrays" << endl;
   test_init2<float>();

}

